/*
 * License MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Gesso, an extension for Mozilla Firefox
 *
 * The Initial Developer of the Original Code is Ningjie "Jim" Chen.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the License, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the License, the GPL or the LGPL.
 *
 * See MPL.txt for terms of the Mozilla Public License Version 1.1
 * See GPL.txt for terms of the GNU General Public License Version 2.0
 * See LGPL.txt for terms of the GNU Lesser General Public License Version 2.1
 */

var vGesso;

function GessoDebug(msg) {  
    var consoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);
    consoleService.logStringMessage(msg);
}

function GessoLoadHandler(o) {
    this.handleEvent = function (e) {
        return GessoObserver.onload(o, this);
    }
}

var GessoObserver = {

    onload: function(win, handler) {
        GessoDebug("Gesso: Window " + win.location + " opened.");
        try { // loadOverlay fails sometimes when called consecutively (bug fixed?)
            var doc = win.document.QueryInterface(Components.interfaces.nsIDOMXULDocument);
            doc.loadOverlay("chrome://gessotsf/content/gesso.xul", null);
        } catch (e) {
            var loader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
            loader.loadSubScript("chrome://gessotsf/content/gesso.js", win)
        }
        win.removeEventListener("load", handler, false);
    },

    observe: function(subj, topic, data) {
    
        if (topic == "domwindowopened") {
            subj.addEventListener("load", new GessoLoadHandler(subj), false);
            
        } else if (topic == "domwindowclosed") {
            GessoDebug("Gesso: Window " + subj.location + " closed.");
            
        } else if (topic == "app-startup") {
            vGesso = Components.classes["@gesso.mozdev.org/gesso;1"].getService(Components.interfaces.iGesso);
            if (vGesso.init()) {
                var vWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
                vWatcher.registerNotification(GessoObserver);
                var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
                observerService.addObserver(GessoObserver, "quit-application", false);
            }
            
        } else if (topic == "quit-application") {
            if (vGesso) {
                var vWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"].getService(Components.interfaces.nsIWindowWatcher);
                vWatcher.unregisterNotification(GessoObserver);
                var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
                observerService.removeObserver(GessoObserver, "quit-application");
                vGesso.term();
                vGesso = null;
            }
        }
    },
    
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.nsIObserver) ||
            iid.equals(Components.interfaces.nsISupports))
            return this;
        
        Components.returnCode = Components.results.NS_ERROR_NO_INTERFACE;
        return null;
    }
};




var GessoWatcher = {

    CID: Components.ID("{0189B1B3-1E17-4f02-BE6C-E9F83D7E5662}"),
    ProgID: "@gesso.mozdev.org/gessoWatcher;1",
    Name: "GessoWatcher",
    Factory: {
        QueryInterface: function (aIID) {
            if (!aIID.equals(Components.interfaces.nsISupports) &&
                !aIID.equals(Components.interfaces.nsIFactory))
                throw Components.results.NS_ERROR_NO_INTERFACE;
            return this;
        },
        
        createInstance: function (outer, iid) {
            if (outer != null)
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            return GessoObserver.QueryInterface(iid);
        }
    },

    registerSelf: function (compMgr, fileSpec, location, type) {
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(this.CID, this.Name, this.ProgID, fileSpec, location, type);
        var catMgr = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
        catMgr.addCategoryEntry("app-startup", this.Name, "service," + this.ProgID, true, true);
    },
    
    unregisterSelf: function (compMgr, fileSpec, location) {
        var catMgr = Components.classes["@mozilla.org/categorymanager;1"].getService(Components.interfaces.nsICategoryManager);
        catMgr.deleteCategoryEntry("app-startup", this.Name, true);
        var compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(this.CID, fileSpec);
            
    },

    getClassObject: function (compMgr, cid, iid) {
        if (cid.equals(this.CID) && iid.equals(Components.interfaces.nsIFactory))
            return this.Factory;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    canUnload: function(compMgr) {
        return vGesso == null;
    },
    
    QueryInterface: function (aIID) {
        if (!aIID.equals(Components.interfaces.nsISupports) &&
            !aIID.equals(Components.interfaces.nsIModule))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
};

function NSGetModule(compMgr, fileSpec) {
    return GessoWatcher;
}

